@@ -946,15 +946,14 @@ void QgsLayoutMapWidget::toggleAtlasScalingOptionsByLayerType()
   if ( QgsWkbTypes::geometryType( layer->wkbType() ) == QgsWkbTypes::PointGeometry )
   {
     //For point layers buffer setting makes no sense, so set "fixed scale" on and disable margin control
-    mAtlasFixedScaleRadio->setChecked( true );
+    if ( mMapItem->atlasScalingMode() == QgsLayoutItemMap::Auto )
+      mAtlasFixedScaleRadio->setChecked( true );
     mAtlasMarginRadio->setEnabled( false );
-    mAtlasPredefinedScaleRadio->setEnabled( false );
   }
   else
   {
     //Not a point layer, so enable changes to fixed scale control
     mAtlasMarginRadio->setEnabled( true );
-    mAtlasPredefinedScaleRadio->setEnabled( true );
   }
 }
 
