@@ -118,8 +118,7 @@ QgsWFSProvider::QgsWFSProvider( const QString &uri, const ProviderOptions &optio
     return;
   }
 
-  //Failed to detect feature type from describeFeatureType -> get first feature from layer to detect type
-  if ( mShared->mWKBType == QgsWkbTypes::Unknown )
+  const auto GetGeometryTypeFromOneFeature = [&]()
   {
     const bool requestMadeFromMainThread = QThread::currentThread() == QApplication::instance()->thread();
     auto downloader = qgis::make_unique<QgsFeatureDownloader>();
@@ -138,6 +137,22 @@ QgsWFSProvider::QgsWFSProvider( const QString &uri, const ProviderOptions &optio
     }
     downloader->run( false, /* serialize features */
                      1 /* maxfeatures */ );
+  };
+
+  //Failed to detect feature type from describeFeatureType -> get first feature from layer to detect type
+  if ( mShared->mWKBType == QgsWkbTypes::Unknown )
+  {
+    GetGeometryTypeFromOneFeature();
+
+    // If we still didn't get the geometry type, and have a filter, temporarily
+    // disable the filter.
+    // See https://github.com/qgis/QGIS/issues/43950
+    if ( mShared->mWKBType == QgsWkbTypes::Unknown && !mSubsetString.isEmpty() )
+    {
+      const QString oldFilter = mShared->setWFSFilter( QString() );
+      GetGeometryTypeFromOneFeature();
+      mShared->setWFSFilter( oldFilter );
+    }
   }
 }
 
@@ -487,6 +502,12 @@ bool QgsWFSProvider::processSQL( const QString &sqlString, QString &errorMsg, QS
     if ( typeName == mShared->mURI.typeName() )
       layerProperties.mSRSName = mShared->srsName();
 
+    if ( typeName.contains( ':' ) )
+    {
+      layerProperties.mNamespaceURI = mShared->mCaps.getNamespaceForTypename( typeName );
+      layerProperties.mNamespacePrefix = QgsWFSUtils::nameSpacePrefix( typeName );
+    }
+
     mShared->mLayerPropertiesList << layerProperties;
   }
 
@@ -1811,9 +1832,9 @@ bool QgsWFSProvider::getCapabilities()
 
     const QgsWfsCapabilities::Capabilities caps = getCapabilities.capabilities();
     mShared->mCaps = caps;
-    mShared->mURI.setGetEndpoints( caps.operationGetEndpoints );
-    mShared->mURI.setPostEndpoints( caps.operationPostEndpoints );
   }
+  mShared->mURI.setGetEndpoints( mShared->mCaps.operationGetEndpoints );
+  mShared->mURI.setPostEndpoints( mShared->mCaps.operationPostEndpoints );
 
   mShared->mWFSVersion = mShared->mCaps.version;
   if ( mShared->mURI.maxNumFeatures() > 0 && mShared->mCaps.maxFeatures > 0 && !( mShared->mCaps.supportsPaging && mShared->mURI.pagingEnabled() ) )
