@@ -28,15 +28,23 @@ from qgis.server import QgsServerRequest
 
 from qgis.testing import unittest
 from qgis.PyQt.QtCore import QSize
-from qgis.core import QgsVectorLayer
+from qgis.core import (
+    QgsVectorLayer,
+    QgsFeatureRequest,
+    QgsExpression,
+    QgsCoordinateReferenceSystem,
+    QgsCoordinateTransform,
+    QgsCoordinateTransformContext,
+    QgsGeometry,
+)
 
 import osgeo.gdal  # NOQA
 
 from test_qgsserver import QgsServerTestBase
 
 # Strip path and content length because path may vary
-RE_STRIP_UNCHECKABLE = b'MAP=[^"]+|Content-Length: \d+|timeStamp="[^"]+"'
-RE_ATTRIBUTES = b'[^>\s]+=[^>\s]+'
+RE_STRIP_UNCHECKABLE = br'MAP=[^"]+|Content-Length: \d+|timeStamp="[^"]+"'
+RE_ATTRIBUTES = br'[^>\s]+=[^>\s]+'
 
 
 class TestQgsServerWFS(QgsServerTestBase):
@@ -96,11 +104,13 @@ class TestQgsServerWFS(QgsServerTestBase):
 
     def test_project_wfs(self):
         """Test some WFS request"""
+
         for request in ('GetCapabilities', 'DescribeFeatureType'):
             self.wfs_request_compare(request)
             self.wfs_request_compare(request, '1.0.0')
 
     def wfs_getfeature_compare(self, requestid, request):
+
         project = self.testdata_path + "test_project_wfs.qgs"
         assert os.path.exists(project), "Project file not found: " + project
 
@@ -109,7 +119,7 @@ class TestQgsServerWFS(QgsServerTestBase):
         header, body = self._execute_request(query_string)
 
         if requestid == 'hits':
-            body = re.sub(b'timeStamp="\d+-\d+-\d+T\d+:\d+:\d+"',
+            body = re.sub(br'timeStamp="\d+-\d+-\d+T\d+:\d+:\d+"',
                           b'timeStamp="****-**-**T**:**:**"', body)
 
         self.result_compare(
@@ -122,6 +132,7 @@ class TestQgsServerWFS(QgsServerTestBase):
         )
 
     def test_getfeature(self):
+
         tests = []
         tests.append(('nobbox', 'GetFeature&TYPENAME=testlayer'))
         tests.append(
@@ -139,6 +150,7 @@ class TestQgsServerWFS(QgsServerTestBase):
 
     def test_wfs_getcapabilities_100_url(self):
         """Check that URL in GetCapabilities response is complete"""
+
         # empty url in project
         project = os.path.join(
             self.testdata_path, "test_project_without_urls.qgs")
@@ -190,6 +202,7 @@ class TestQgsServerWFS(QgsServerTestBase):
                     "onlineResource=\"my_wfs_advertised_url\"" in item, True)
 
     def result_compare(self, file_name, error_msg_header, header, body):
+
         self.assert_headers(header, body)
         response = header + body
         reference_path = self.testdata_path + file_name
@@ -203,6 +216,7 @@ class TestQgsServerWFS(QgsServerTestBase):
                             (error_msg_header))
 
     def wfs_getfeature_post_compare(self, requestid, request):
+
         project = self.testdata_path + "test_project_wfs.qgs"
         assert os.path.exists(project), "Project file not found: " + project
 
@@ -257,6 +271,62 @@ class TestQgsServerWFS(QgsServerTestBase):
 """
         tests.append(('srsname_post', srsTemplate.format("")))
 
+        # Issue https://github.com/qgis/QGIS/issues/36398
+        # Check get feature within polygon having srsName=EPSG:4326 (same as the project/layer)
+        within4326FilterTemplate = """<?xml version="1.0" encoding="UTF-8"?>
+<wfs:GetFeature service="WFS" version="1.0.0" {} xmlns:wfs="http://www.opengis.net/wfs" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.opengis.net/wfs http://schemas.opengis.net/wfs/1.1.0/wfs.xsd">
+  <wfs:Query typeName="testlayer" srsName="EPSG:4326" xmlns:feature="http://www.qgis.org/gml">
+    <ogc:Filter xmlns:ogc="http://www.opengis.net/ogc">
+      <Within>
+        <PropertyName>geometry</PropertyName>
+        <Polygon xmlns="http://www.opengis.net/gml" srsName="EPSG:4326">
+          <exterior>
+            <LinearRing>
+              <posList srsDimension="2">
+                8.20344131 44.90137909
+                8.20347748 44.90137909
+                8.20347748 44.90141005
+                8.20344131 44.90141005
+                8.20344131 44.90137909
+              </posList>
+            </LinearRing>
+          </exterior>
+        </Polygon>
+      </Within>
+    </ogc:Filter>
+  </wfs:Query>
+</wfs:GetFeature>
+"""
+        tests.append(('within4326FilterTemplate_post', within4326FilterTemplate.format("")))
+
+        # Check get feature within polygon having srsName=EPSG:3857 (different from the project/layer)
+        # The coordinates are converted from the one in 4326
+        within3857FilterTemplate = """<?xml version="1.0" encoding="UTF-8"?>
+<wfs:GetFeature service="WFS" version="1.0.0" {} xmlns:wfs="http://www.opengis.net/wfs" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.opengis.net/wfs http://schemas.opengis.net/wfs/1.1.0/wfs.xsd">
+  <wfs:Query typeName="testlayer" srsName="EPSG:3857" xmlns:feature="http://www.qgis.org/gml">
+    <ogc:Filter xmlns:ogc="http://www.opengis.net/ogc">
+      <Within>
+        <PropertyName>geometry</PropertyName>
+        <Polygon xmlns="http://www.opengis.net/gml" srsName="EPSG:3857">
+          <exterior>
+            <LinearRing>
+              <posList srsDimension="2">
+              913202.90938171 5606008.98136456
+              913206.93580769 5606008.98136456
+              913206.93580769 5606013.84701639
+              913202.90938171 5606013.84701639
+              913202.90938171 5606008.98136456
+              </posList>
+            </LinearRing>
+          </exterior>
+        </Polygon>
+      </Within>
+    </ogc:Filter>
+  </wfs:Query>
+</wfs:GetFeature>
+"""
+        tests.append(('within3857FilterTemplate_post', within3857FilterTemplate.format("")))
+
         srsTwoLayersTemplate = """<?xml version="1.0" encoding="UTF-8"?>
 <wfs:GetFeature service="WFS" version="1.0.0" {} xmlns:wfs="http://www.opengis.net/wfs" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.opengis.net/wfs http://schemas.opengis.net/wfs/1.1.0/wfs.xsd">
   <wfs:Query typeName="testlayer" srsName="EPSG:3857" xmlns:feature="http://www.qgis.org/gml">
@@ -432,6 +502,7 @@ class TestQgsServerWFS(QgsServerTestBase):
 
     def test_getFeatureFeatureId(self):
         """Test GetFeature with featureid"""
+
         self.wfs_request_compare(
             "GetFeature", '1.0.0', "SRSNAME=EPSG:4326&TYPENAME=testlayer&FEATUREID=testlayer.0", 'wfs_getFeature_1_0_0_featureid_0')
 
@@ -572,6 +643,120 @@ class TestQgsServerWFS(QgsServerTestBase):
                 + "&SRSNAME=EPSG:4326&TYPENAME=testlayer&FEATUREID=testlayer.0",
                 'wfs_getFeature_1_0_0_featureid_0_json')
 
+    def test_insert_srsName(self):
+        """Test srsName is respected when insering"""
+
+        post_data = """
+        <Transaction xmlns="http://www.opengis.net/wfs" xsi:schemaLocation="http://www.qgis.org/gml http://localhost:8000/?SERVICE=WFS&amp;REQUEST=DescribeFeatureType&amp;VERSION=1.0.0&amp;TYPENAME=as_symbols" service="WFS" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" version="{version}" xmlns:gml="http://www.opengis.net/gml">
+            <Insert xmlns="http://www.opengis.net/wfs">
+                <as_symbols xmlns="http://www.qgis.org/gml">
+                <name xmlns="http://www.qgis.org/gml">{name}</name>
+                <geometry xmlns="http://www.qgis.org/gml">
+                    <gml:Point srsName="{srsName}">
+                    <gml:coordinates cs="," ts=" ">{coordinates}</gml:coordinates>
+                    </gml:Point>
+                </geometry>
+                </as_symbols>
+            </Insert>
+        </Transaction>
+        """
+
+        project = self.testdata_path + \
+            "test_project_wms_grouped_layers.qgs"
+        assert os.path.exists(project), "Project file not found: " + project
+
+        query_string = '?SERVICE=WFS&MAP={}'.format(
+            urllib.parse.quote(project))
+        request = post_data.format(
+            name='4326-test1',
+            version='1.1.0',
+            srsName='EPSG:4326',
+            coordinates='10.67,52.48'
+        )
+        header, body = self._execute_request(
+            query_string, requestMethod=QgsServerRequest.PostMethod, data=request.encode('utf-8'))
+
+        # Verify
+        vl = QgsVectorLayer(self.testdata_path + 'test_project_wms_grouped_layers.gpkg|layername=as_symbols', 'as_symbols')
+        self.assertTrue(vl.isValid())
+        feature = next(vl.getFeatures(QgsFeatureRequest(QgsExpression('"name" = \'4326-test1\''))))
+        geom = feature.geometry()
+
+        tr = QgsCoordinateTransform(QgsCoordinateReferenceSystem.fromEpsgId(4326), vl.crs(), QgsCoordinateTransformContext())
+
+        geom_4326 = QgsGeometry.fromWkt('point( 10.67 52.48)')
+        geom_4326.transform(tr)
+        self.assertEqual(geom.asWkt(0), geom_4326.asWkt(0))
+
+        # Now: insert a feature in layer's CRS
+        request = post_data.format(
+            name='25832-test1',
+            version='1.1.0',
+            srsName='EPSG:25832',
+            coordinates='613412,5815738'
+        )
+        header, body = self._execute_request(
+            query_string, requestMethod=QgsServerRequest.PostMethod, data=request.encode('utf-8'))
+
+        feature = next(vl.getFeatures(QgsFeatureRequest(QgsExpression('"name" = \'25832-test1\''))))
+        geom = feature.geometry()
+        self.assertEqual(geom.asWkt(0), geom_4326.asWkt(0))
+
+        # Tests for inverted axis issue GH #36584
+        # Cleanup
+        self.assertTrue(vl.startEditing())
+        vl.selectByExpression('"name" LIKE \'4326-test%\'')
+        vl.deleteSelectedFeatures()
+        self.assertTrue(vl.commitChanges())
+
+        self.i = 0
+
+        def _test(version, srsName, lat_lon=False):
+            self.i += 1
+            name = '4326-test_%s' % self.i
+            request = post_data.format(
+                name=name,
+                version=version,
+                srsName=srsName,
+                coordinates='52.48,10.67' if lat_lon else '10.67,52.48'
+            )
+            header, body = self._execute_request(
+                query_string, requestMethod=QgsServerRequest.PostMethod, data=request.encode('utf-8'))
+            feature = next(vl.getFeatures(QgsFeatureRequest(QgsExpression('"name" = \'%s\'' % name))))
+            geom = feature.geometry()
+            self.assertEqual(geom.asWkt(0), geom_4326.asWkt(0), "Failed: %s , %s, lat_lon=%s" % (version, srsName, lat_lon))
+
+        _test('1.1.0', 'urn:ogc:def:crs:EPSG::4326', lat_lon=True)
+        _test('1.1.0', 'http://www.opengis.net/def/crs/EPSG/0/4326', lat_lon=True)
+        _test('1.1.0', 'http://www.opengis.net/gml/srs/epsg.xml#4326', lat_lon=False)
+        _test('1.1.0', 'EPSG:4326', lat_lon=False)
+
+        _test('1.0.0', 'urn:ogc:def:crs:EPSG::4326', lat_lon=True)
+        _test('1.0.0', 'http://www.opengis.net/def/crs/EPSG/0/4326', lat_lon=True)
+        _test('1.0.0', 'http://www.opengis.net/gml/srs/epsg.xml#4326', lat_lon=False)
+        _test('1.0.0', 'EPSG:4326', lat_lon=False)
+
+        def _test_getFeature(version, srsName, lat_lon=False):
+            # Now get the feature through WFS using BBOX filter
+            bbox = QgsGeometry.fromWkt('point( 10.7006 52.4317)').boundingBox()
+            bbox.grow(0.0001)
+            bbox_text = "%s,%s,%s,%s" % ((bbox.yMinimum(), bbox.xMinimum(), bbox.yMaximum(), bbox.xMaximum()) if lat_lon else (bbox.xMinimum(), bbox.yMinimum(), bbox.xMaximum(), bbox.yMaximum()))
+            req = query_string + '&REQUEST=GetFeature&VERSION={version}&TYPENAME=as_symbols&SRSNAME={srsName}&BBOX={bbox},{srsName}'.format(version=version, srsName=srsName, bbox=bbox_text)
+            header, body = self._execute_request(req)
+            self.assertTrue(b'gid>7' in body, "GetFeature Failed: %s , %s, lat_lon=%s" % (version, srsName, lat_lon))
+
+        _test_getFeature('1.1.0', 'urn:ogc:def:crs:EPSG::4326', lat_lon=True)
+        _test_getFeature('1.1.0', 'EPSG:4326', lat_lon=False)
+
+        _test_getFeature('1.0.0', 'urn:ogc:def:crs:EPSG::4326', lat_lon=True)
+        _test_getFeature('1.0.0', 'EPSG:4326', lat_lon=False)
+
+        # Cleanup
+        self.assertTrue(vl.startEditing())
+        vl.selectByExpression('"name" LIKE \'4326-test%\'')
+        vl.deleteSelectedFeatures()
+        self.assertTrue(vl.commitChanges())
+
 
 if __name__ == '__main__':
     unittest.main()
